==> PyBookReader-0.4.4.tar.gz: PyBookReader-0.4.4/pybookreader/bookmarks.py <==
#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os
import shelve

from pybookreader.utils import *

##----------------------------------------------------------------------
##----------------------------------------------------------------------


class Bookmarks:

    def __init__(self):

        self.categories_list = {}

        if not os.path.exists(bookmarks_file): return

        bookmarks_db = shelve.open(bookmarks_file)
        self.categories_list = bookmarks_db['categories_list']

        # for backward compatible
        for c in self.categories_list:
            for b in self.categories_list[c]:
                if not hasattr(b, 'externalfilter'):
                    setattr(b, 'externalfilter', None)
                if not hasattr(b, 'internalfilters'):
                    setattr(b, 'internalfilters', None)

        bookmarks_db.close()

##----------------------------------------------------------------------

    def make_new_category(self, name):
        if not self.categories_list.has_key(name) and len(name) > 0:
            self.categories_list[name] = []

##----------------------------------------------------------------------

    def make_new_book(self, filename):

        book=Book()
        book.filename = filename

        self.make_new_category('Default')
        self.categories_list['Default'].append(book)

        return book

##----------------------------------------------------------------------

    def add_new_book(self, book):

        self.make_new_category('Default')
        self.categories_list['Default'].append(book)

        return book

##----------------------------------------------------------------------

    def move_book(self, filename, new_cat):
        '''move book to new category'''

        for cat in self.categories_list:
            n = 0
            found = 0
            for b in self.categories_list[cat]:
                if b.filename == filename:
                    book = b
                    del self.categories_list[cat][n]
                    found = 1
                    break
                n = n+1
            if found: break

        if found:
            self.categories_list[new_cat].append(book)

        return found

##----------------------------------------------------------------------

    def remove_book(self, filename):

        # remove from categories_list
        for cat in self.categories_list:
            n = 0
            found = 0
            for b in self.categories_list[cat]:
                if b.filename == filename:
                    book = b
                    del self.categories_list[cat][n]
                    found = 1
                    break
                n = n+1
            if found: break


        return found

##----------------------------------------------------------------------

    def remove_bookmark(self, filename, mark_number):
        found = 0
        for c in self.categories_list:
            for b in self.categories_list[c]:
                if b.filename == filename:
                    n = 0
                    for m in b.marks_list:
                        if m.number == mark_number:
                            del b.marks_list[n]
                            found = 1
                            break
                        n = n+1
        return found

##----------------------------------------------------------------------

    def get_book(self, filename):
        for c in self.categories_list:
            for b in self.categories_list[c]:
                if b.filename == filename: return b
        return None

##----------------------------------------------------------------------

    def get_any_book(self):
        for c in self.categories_list:
            if len(self.categories_list[c]) == 0:
                continue
            else:
                return self.categories_list[c][0]
        return None

##----------------------------------------------------------------------

    def get_mark(self, book, mark_number):
        for m in book.marks_list:
            if m.number == mark_number: return m
        return None


##----------------------------------------------------------------------

    def save_bookmarks(self):

        bookmarks_db = shelve.open(bookmarks_file)
        bookmarks_db['categories_list'] = self.categories_list
        bookmarks_db.close()

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class Book:

    def __init__(self):

        #self.category = 'Default'
        self.filename = None # abs path
        self.author = ''
        self.name = ''
        self.description = None
        self.current_position = 0.0
        self.current_offset = 0
        self.encoding = 'koi8-r' # FIXME
        self.file_type = 'plain_text'
        self.internalfilters = None
        self.externalfilter = None
        self.marks_list = []

    def add_bookmark(self, position, offset, description):

        mark = _Mark()
        mark.position = position
        mark.offset = offset
        mark.description = description

        # search uniq number
        num = 0
        while 1:
            found = 0
            for i in self.marks_list:
                if i.number == num:
                    found = 1
                    break
            if not found:
                break
            num = num+1

        mark.number = num
        self.marks_list.append(mark)

        # sort by position
        self.marks_list.sort(lambda x, y: cmp(x.position, y.position))

        return mark

##----------------------------------------------------------------------

    def get_title(self):
        if self.author and self.name:
            name = self.author + ' - ' +  self.name
        elif self.author:
            name = self.author
        elif self.name:
            name = self.name
        else:
            name = self.filename
        return name

##----------------------------------------------------------------------

    def get_mark_at_offset(self, offset):
        for m in self.marks_list:
            if offset >= m.offset and offset <= m.offset+20: #FIXME: length
                return m
        return None

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class Category:

    def __init__(self):

        self.name = 'Default'
        self.description = ''

##----------------------------------------------------------------------

class _Mark:

    def __init__(self):

        self.number = 0
        self.position = 0.0
        self.description = ''

        self.author = ''
        self.name = ''

        self.offset = 0


##----------------------------------------------------------------------
##----------------------------------------------------------------------

if(__name__=="__main__"):

    bookmarks = Bookmarks()

    print '****** books_list ******'
    print

    for book in bookmarks.books_list:

        print 'category =', book.category
        print 'filename =', book.filename
        print 'author =', book.author
        print 'name =', book.name
        print 'description =', book.description
        print 'current_position =', book.current_position
        print 'encoding =', book.encoding
        print

        print '   *** marks_list ***'
        print

        for mark in book.marks_list:

            print '   namber =', mark.number
            print '   offset =', mark.offset
            print '   position =', mark.position
            print '   description =', mark.description
            print '   author =', mark.author
            print '   name =', mark.name
            print

==> PyBookReader-0.4.5.tar.gz: PyBookReader-0.4.5/pybookreader/bookmarks.py <==
#!/usr/bin/env python
# -*- mode: python; coding: koi8-r -*-


import string, os
import shelve

from utils import *

##----------------------------------------------------------------------
##----------------------------------------------------------------------


class Bookmarks:

    def __init__(self):

        self.categories_list = {}

        if not os.path.exists(bookmarks_file): return

        bookmarks_db = shelve.open(bookmarks_file)
        self.categories_list = bookmarks_db['categories_list']

        bookmarks_db.close()

##----------------------------------------------------------------------

    def make_new_category(self, name):
        if not self.categories_list.has_key(name) and len(name) > 0:
            self.categories_list[name] = []

##----------------------------------------------------------------------

    def make_new_book(self, filename):

        book=Book()
        book.filename = filename

        self.make_new_category('Default')
        self.categories_list['Default'].append(book)

        return book

##----------------------------------------------------------------------

    def add_new_book(self, book):

        self.make_new_category('Default')
        self.categories_list['Default'].append(book)

        return book

##----------------------------------------------------------------------

    def move_book(self, filename, new_cat):
        '''move book to new category'''

        for cat in self.categories_list:
            n = 0
            found = 0
            for b in self.categories_list[cat]:
                if b.filename == filename:
                    book = b
                    del self.categories_list[cat][n]
                    found = 1
                    break
                n = n+1
            if found: break

        if found:
            self.categories_list[new_cat].append(book)

        return found

##----------------------------------------------------------------------

    def remove_book(self, filename):

        # remove from categories_list
        for cat in self.categories_list:
            n = 0
            found = 0
            for b in self.categories_list[cat]:
                if b.filename == filename:
                    book = b
                    del self.categories_list[cat][n]
                    found = 1
                    break
                n = n+1
            if found: break


        return found

##----------------------------------------------------------------------

    def remove_bookmark(self, filename, mark_number):
        found = 0
        for c in self.categories_list:
            for b in self.categories_list[c]:
                if b.filename == filename:
                    n = 0
                    for m in b.marks_list:
                        if m.number == mark_number:
                            del b.marks_list[n]
                            found = 1
                            break
                        n = n+1
        return found

##----------------------------------------------------------------------

    def get_book(self, filename):
        for c in self.categories_list:
            for b in self.categories_list[c]:
                if b.filename == filename: return b
        return None

##----------------------------------------------------------------------

    def get_any_book(self):
        for c in self.categories_list:
            if len(self.categories_list[c]) == 0:
                continue
            else:
                return self.categories_list[c][0]
        return None

##----------------------------------------------------------------------

    def get_mark(self, book, mark_number):
        for m in book.marks_list:
            if m.number == mark_number: return m
        return None


##----------------------------------------------------------------------

    def save_bookmarks(self):

        bookmarks_db = shelve.open(bookmarks_file)
        bookmarks_db['categories_list'] = self.categories_list
        bookmarks_db.close()

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class Book:

    def __init__(self):

        #self.category = 'Default'
        self.filename = None # abs path
        self.author = ''
        self.name = ''
        self.description = None
        self.current_position = 0.0
        self.current_offset = 0
        self.encoding = 'koi8-r' # FIXME
        self.file_type = 'plain_text'
        self.internalfilters = None
        self.externalfilter = None
        self.marks_list = []

        self.exclusive_view = 0
        self.font = None
        self.fg_color = None
        self.bg_color = None
        self.bm_color = None


    def __setstate__(self, state): # for backward compatible

        if 'externalfilter' not in state:
            self.externalfilter = None
            self.internalfilters = None

        if 'exclusive_view' not in state:
            self.exclusive_view = 0
            self.font = None
            self.fg_color = None
            self.bg_color = None
            self.bm_color = None

        self.__dict__.update(state)


    def add_bookmark(self, position, offset, description):

        mark = _Mark()
        mark.position = position
        mark.offset = offset
        mark.description = description

        # search uniq number
        num = 0
        while 1:
            found = 0
            for i in self.marks_list:
                if i.number == num:
                    found = 1
                    break
            if not found:
                break
            num = num+1

        mark.number = num
        self.marks_list.append(mark)

        # sort by position
        self.marks_list.sort(lambda x, y: cmp(x.position, y.position))

        return mark

##----------------------------------------------------------------------

    def get_title(self):
        if self.author and self.name:
            name = self.author + ' - ' +  self.name
        elif self.author:
            name = self.author
        elif self.name:
            name = self.name
        else:
            name = fix_filename(self.filename)
        return name

##----------------------------------------------------------------------

    def get_mark_at_offset(self, offset):
        for m in self.marks_list:
            if offset >= m.offset and offset <= m.offset+20: #FIXME: length
                return m
        return None

##----------------------------------------------------------------------
##----------------------------------------------------------------------

class Category:

    def __init__(self):

        self.name = 'Default'
        self.description = ''

##----------------------------------------------------------------------

class _Mark:

    def __init__(self):

        self.number = 0
        self.position = 0.0
        self.description = ''

        self.author = ''
        self.name = ''

        self.offset = 0


##----------------------------------------------------------------------
##----------------------------------------------------------------------

if(__name__=="__main__"):

    bookmarks = Bookmarks()

    print '****** books_list ******'
    print

    for book in bookmarks.books_list:

        print 'category =', book.category
        print 'filename =', book.filename
        print 'author =', book.author
        print 'name =', book.name
        print 'description =', book.description
        print 'current_position =', book.current_position
        print 'encoding =', book.encoding
        print

        print '   *** marks_list ***'
        print

        for mark in book.marks_list:

            print '   namber =', mark.number
            print '   offset =', mark.offset
            print '   position =', mark.position
            print '   description =', mark.description
            print '   author =', mark.author
            print '   name =', mark.name
            print
